@@ -29,19 +29,27 @@ use url::Url;
 struct CacheKey {
     ak: Box<str>,
     bucket: Box<str>,
+    hosts_crc32: u32,
 }
 
 impl CacheKey {
     #[inline]
-    fn new(ak: Box<str>, bucket: Box<str>) -> Self {
-        Self { ak, bucket }
+    fn new(ak: Box<str>, bucket: Box<str>, hosts_crc32: u32) -> Self {
+        Self {
+            ak,
+            bucket,
+            hosts_crc32,
+        }
     }
 }
 
 impl Serialize for CacheKey {
     #[inline]
     fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        s.collect_str(&format!("{}:{}", self.ak, self.bucket))
+        s.collect_str(&format!(
+            "cache-key-v2:{}:{}:{}",
+            self.ak, self.bucket, self.hosts_crc32
+        ))
     }
 }
 
@@ -56,13 +64,26 @@ impl<'de> Visitor<'de> for CacheKeyVisitor {
     }
 
     fn visit_str<E: DeError>(self, value: &str) -> Result<Self::Value, E> {
-        let mut iter = value.splitn(2, ':');
-        match (iter.next(), iter.next()) {
-            (Some(ak), Some(bucket)) => Ok(CacheKey {
-                ak: ak.into(),
-                bucket: bucket.into(),
-            }),
-            _ => Err(E::custom(format!("Invalid cache_key: {}", value))),
+        if let Some(value) = value.strip_prefix("cache-key-v2:") {
+            let mut iter = value.splitn(3, ':');
+            match (iter.next(), iter.next(), iter.next()) {
+                (Some(ak), Some(bucket), Some(crc32_str)) => Ok(CacheKey {
+                    ak: ak.into(),
+                    bucket: bucket.into(),
+                    hosts_crc32: crc32_str.parse().map_err(|err| {
+                        E::custom(format!(
+                            "Cannot parse hosts_crc32 from cache_key: {}: {}",
+                            value, err
+                        ))
+                    })?,
+                }),
+                _ => Err(E::custom(format!("Invalid cache_key: {}", value))),
+            }
+        } else {
+            Err(E::custom(format!(
+                "Unrecognized version of cache_key: {}",
+                value
+            )))
         }
     }
 }
@@ -136,8 +157,8 @@ impl HostsQuerier {
     ) -> IOResult<Vec<String>> {
         Lazy::force(&CACHE_INIT);
 
-        let response_body = self.query_for_domains(ak, bucket)?;
-        return Ok(response_body
+        Ok(self
+            .query_for_domains(ak, bucket)?
             .hosts
             .first()
             .expect("No host in uc query v4 response body")
@@ -145,22 +166,11 @@ impl HostsQuerier {
             .domains
             .iter()
             .map(|domain| normalize_domain(domain, use_https))
-            .collect());
-
-        #[inline]
-        fn normalize_domain(domain: &str, use_https: bool) -> String {
-            if domain.contains("://") {
-                domain.to_string()
-            } else if use_https {
-                "https://".to_owned() + domain
-            } else {
-                "http://".to_owned() + domain
-            }
-        }
+            .collect())
     }
 
     fn query_for_domains(&self, ak: &str, bucket: &str) -> IOResult<ResponseBody> {
-        let cache_key = CacheKey::new(ak.into(), bucket.into());
+        let cache_key = CacheKey::new(ak.into(), bucket.into(), self.uc_selector.all_hosts_crc32());
 
         let mut modified = false;
         let cache_value = CACHE_MAP
@@ -235,6 +245,7 @@ fn query_for_domains_without_cache(
                 ak.as_ref(),
                 bucket.as_ref()
             );
+            let use_https = parse_url_protocol(host).unwrap_or(false);
 
             let url = Url::parse_with_params(
                 &format!("{}/v4/query", host),
@@ -275,7 +286,7 @@ fn query_for_domains_without_cache(
                             host.uc
                                 .domains
                                 .iter()
-                                .map(|domain| domain.to_string())
+                                .map(|domain| normalize_domain(domain, use_https))
                                 .collect()
                         })
                         .expect("No host in uc query v4 response body");
@@ -390,9 +401,9 @@ fn save_cache() -> IOResult<()> {
         return Ok(());
     }
     if let Err(err) = _save_cache(&cache_tempfile_path) {
-        warn!("Failed to save cache {:?}: {}", cache_file_path, err);
+        warn!("Failed to save cache {:?}: {}", cache_tempfile_path, err);
     } else {
-        info!("Save cache to {:?} successfully", cache_file_path);
+        info!("Save cache to {:?} successfully", cache_tempfile_path);
         if let Err(err) = rename_file(&cache_tempfile_path, &cache_file_path) {
             warn!(
                 "Failed to move cache file from {:?} to {:?}: {}",
@@ -420,6 +431,28 @@ fn save_cache() -> IOResult<()> {
     }
 }
 
+#[inline]
+fn normalize_domain(domain: &str, use_https: bool) -> String {
+    if domain.contains("://") {
+        domain.to_string()
+    } else if use_https {
+        "https://".to_owned() + domain
+    } else {
+        "http://".to_owned() + domain
+    }
+}
+
+#[inline]
+fn parse_url_protocol(url: &str) -> Option<bool> {
+    if url.starts_with("https://") {
+        Some(true)
+    } else if url.starts_with("http://") {
+        Some(false)
+    } else {
+        None
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{
@@ -564,8 +597,11 @@ mod tests {
                     HostsQuerier::new(host_selector, 1, dotter, Timeouts::default_http_client());
                 let io_urls = querier.query_for_io_urls(ACCESS_KEY, BUCKET_NAME, false)?;
                 assert_eq!(&io_urls, &["http://iovip.qbox.me".to_owned()]);
-                assert_eq!(&querier.uc_selector.hosts(), &["uc.qbox.me".to_owned()]);
-                assert_eq!(&querier.uc_selector.select_host().host, "uc.qbox.me");
+                assert_eq!(
+                    &querier.uc_selector.hosts(),
+                    &["http://uc.qbox.me".to_owned()]
+                );
+                assert_eq!(&querier.uc_selector.select_host().host, "http://uc.qbox.me");
                 sleep(Duration::from_secs(5));
                 assert_eq!(monitor_called.load(Relaxed), 1);
                 Ok(())
